#!/usr/bin/env python
#   Copyright 2012 Gary Wright http://identi.ca/wrightsolutions
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from __future__ import with_statement
from fabric.api import abort, cd, run, settings
from fabric.contrib.console import confirm
from fabric.contrib.files import exists

""" res.succeeded is preferred for newer fabric, but not res.failed
is used where backward compatibility is preferable. """

cmd_wget = "wget -nd -m -l 0 "
cmd_gpg_recv = "gpg --keyserver keys.gnupg.net --recv-keys "
cmd_gpg_verify = "gpg --verify --no-tty --batch "
cmd_gpg_list = "gpg --list-key --no-tty --batch "
dir_fab = "primerange-benchmark.fab"
dirpath_fab = "~/" + dir_fab


def tilde_expand(path):
    if path.startswith('~/'):
        path_expanded = run("dirname ~/.")
    else:
        return path
    return path_expanded


def rmdir_existing(dirpath,ignore_missing=True):
    if dirpath.startswith('~/'):
        """ set function default return to True """
        rmdir_flag = True
    else:
        """ As a safety feature against dangerous rm -fR
        commands, any directory path that does not begin
        with ~/ will be rejected (function returns False) """
        return False
    exists_dirpath = True
    with settings(warn_only=True):
        """ Tilde expansion only applies when '~' is unquoted """
        exists_dirpath = exists(tilde_expand(dirpath))
        if not exists_dirpath:
            return ignore_missing
    if exists_dirpath:
        run("rm -vfR " + dirpath)
    return True


def abort_with_cleanup(dirpath,abort_message,ignore_missing=True):
    rmdir_existing(dirpath,ignore_missing)
    abort(abort_message)
    return


def mkdir_existing(dirpath,remove_existing=True):
    if dirpath.startswith('~/'):
        """ set function default return to True """
        mkdir_flag = True
    else:
        """ As a safety feature against dangerous rm -fR
        commands, any directory path that does not begin
        with ~/ will be rejected (function returns False) """
        return False
    exists_dirpath = True
    with settings(warn_only=True):
        """ Tilde expansion only applies when '~' is unquoted """
        # exists_dirpath = exists(dirpath,verbose=True)
        exists_dirpath = exists(tilde_expand(dirpath))
        #print "exists_dirpath=%s, remove_existing=%s\n" \
        #% (exists_dirpath,remove_existing)
    if exists_dirpath and remove_existing:
        run("rm -vfR " + dirpath)
    mkdir_res = run("mkdir " + dirpath)
    if mkdir_res.failed:
        mkdir_flag = False
    return mkdir_flag


def gpg_recv_and_verify(signed_file):
    gpgres = run(cmd_gpg_list + "CE0222D0EB8CFF0E")
    if gpgres.failed:
       gpgres = run(cmd_gpg_recv + "CE0222D0EB8CFF0E")
       if not gpgres.failed:
       	  abort("Aborting as preparation for gpg verification failed.")
    gpgres = run(cmd_gpg_verify + signed_file + ".sig")	 
    #gpgres = contains(tar,'02 Feb 2012 18:48:46 GMT using DSA key ID EB8C')	 
    return gpgres
 

def primerange_get_and_build(zeros='0',pybin='python'):
    if not pybin.startswith('python'):
       	  abort("Aborting as pybin given is invalid.")
    zeros_added = int(zeros)
    tar = 'primerange-partial__20111008prerelease.tar.gz'
    tar_url = "http://download-mirror.savannah.gnu.org/releases/primerange/%s.sig" % tar
    mkdir_existing(dirpath_fab)
    with settings(warn_only=True):
	 """
	 res = local('curl -I http://www.wrightsolutions.co.uk/',
                     capture=True)
	 """
         with cd(dirpath_fab):
             res = run(cmd_wget + tar_url)
             if not res.failed:
	 	 tar_url = "http://download-mirror.savannah.gnu.org/releases/primerange/%s" % tar
		 res = run(cmd_wget + tar_url)
		 """ capture=True can only be used for local - not okay for run() """
		 if not res.failed:
		    	res = gpg_recv_and_verify(tar)
	
    if res.failed and not confirm("Download of primerange failed. Continue anyway?"):
        abort_with_cleanup(dirpath_fab,"Aborting at user request.")

    run("which gcc pv")

    if (zeros_added < 3):
        print "Timed benchmark run being prepared.\n"
    else:
        print "Timed run being prepared with zeros=%d.\n" % zeros_added

    with cd(dirpath_fab):
        res = run("tar -zxf " + tar)
        setzexpdir = "primerange/setz_exp_small_to_the_gigantic_unopt"
        res = run("cp -p " + setzexpdir + "/setz_exp_small_to_the_gigantic*.{c,py} .")
	setzexpharness = "setz_exp_small_to_the_gigantic_unopt-harness1365.py"
        res = run("sed '/esl.setz_esl_init(1365,/s$(1365,453,$(23205,10,$;" +
		"s$,0)$," + zeros + ")$'" +
		" < " + setzexpdir + "/" + setzexpharness + " > ./" + setzexpharness)

        setzexp = "setz_exp_small_to_the_gigantic_unopt"
        #res = run("gcc " + setzexp + ".c -o " + setzexp + ".o -lm -lgmp")
	res = run(pybin + " ./" + setzexp + "-setup.py build_ext --inplace")
	""" Missing Python.h complaint might require you to 'apt-get install python-dev'
	or 'yum install python-devel' or 'yum install python-devel.x86_64' """

        timed_run_flag = False
        if (zeros_added < 5):
            timed_run_flag = True
        elif (zeros_added < 7):
            if confirm("Timed run may take an hour to complete. Continue?"):
                timed_run_flag = True
        elif confirm("Timed run may take between an hour and 24 hours to complete. Continue?"):
            timed_run_flag = True

        if timed_run_flag:
	        res = run("date;time " + pybin + " ./" + setzexp + "-harness1365.py")
	        #res = run("date;time ./" + setzexp + "-harness1365.py | pv -f -i 60 --buffer-size 1k -b > /dev/null;date" )
		""" fgrep of output will perhaps not show immediately due to buffering / other delay """
		#res = run("date;time ./" + setzexp + ".o | fgrep 'processing begun...';date" )

