#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   Copyright 2014 Gary Wright http://www.wrightsolutions.co.uk/contact
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#   On Debian systems, the complete text of the Apache license can be found in
#   /usr/share/common-licenses/Apache-2.0

# fab -a -f django14skeleton.py -u root -H aa.bb.cc.dd default
# fab -a -f django14skeleton.py -u root -H aa.bb.cc.dd default:repos=true
# fab -a -f django14skeleton.py -u root --port=22222 -H 127.0.0.1 default:repos=true

# Tested against fabric 1.4.3
# http://ftp.uk.debian.org/debian/pool/main/f/fabric/fabric_1.4.3-1.debian.tar.gz

from __future__ import with_statement
from fabric.api import abort, cd, env, get, put, run, settings, sudo
#from fabric.contrib.console import confirm
from fabric.contrib.files import exists,append
from os import path

dir_fab = "django14skeleton.fab"
APP_NAME='app14'
PROJECT_NAME='proj14'
repo_url_django = 'https://bitbucket.org/wrightsolutions/local2django14bench'
repo_url_localbin = 'https://bitbucket.org/wrightsolutions/local-bin'
#repo_url_localwebserver = 'https://bitbucket.org/wrightsolutions/local-webserver'
dirpath_fab = '/tmp'
dirpath_repos = '/tmp'


def kern64():
    with settings(warn_only=True):
        kern64line = "/bin/uname -a | {0} 'x86_64'".format('/bin/fgrep')
        kern64res = run(kern64line)
    if kern64res.failed:
        return False
    return True


def pretty_name():
    pretty_name = 'Unknown!'
    with settings(warn_only=True):
        pretty_grep = "{0} '^PRETTY_NAME' /etc/os-release".format('/bin/egrep')
        pretty_res = run(pretty_grep)

        if pretty_res.succeeded:
            pretty_last = pretty_res.split(chr(61))[-1]
            """ Both types of quotes stripped below """
            pretty_name = pretty_last.strip('"\\\'')

    return pretty_name


def tilde_expand(path_short,abort_on_fail=True):
    if path_short.startswith('~/'):
        """ path_expanded = path.expanduser(path_short)
        will not do as you are interested in the remote user
        not the user account on your local machine! """
        expandres = run("dirname ~/.")
        if expandres.failed:
            if abort_on_fail:
                abort("Failed during path expansion of {0}".format(path_short))
            path_expanded = path.short
        else:
            path_expanded = path_short.replace('~/',"{0}/".format(expandres),1)
        """ path_expanded = run("dirname ~/.") """
    else:
        return path_short
    return path_expanded


def dirpath_dlp(dirpath):
    dlp_res = None
    dlp_line = "/bin/vdir -dlp {0}".format(dirpath)
    with settings(warn_only=True):
        dlp_res = run(dlp_line)
    return dlp_res


def repo_clone(repo_url,revision=None):
    if revision:
        cloneres = run("/usr/bin/hg clone -r {0} {1}".format(revision,repo_url))
    else:
        cloneres = run("/usr/bin/hg clone {0}".format(repo_url))
    if cloneres.failed:
        repo_url = None
    return repo_url


def run_or_sudo(run_cmd,run_always=False):
    if env.user == 'root':
        return run(run_cmd)
    elif run_always:
        """ run_always flag set True means
        that sudo will never be used. """
        return run(run_cmd)
    return sudo(run_cmd)


def rmdir_existing(dirpath,ignore_missing=True,rm_verbose=True):
    """ run_or_sudo() is not always called in preference to plain run()
    so as we do not use sudo unless absolutely necessary.
    Deleting from the users own home directory would not need sudo
    """
    rmdir_flag = False
    if dirpath.startswith('~/'):
        """ set function default return to True """
        rmdir_flag = True
    elif '/djapian/' in dirpath:
        """ set function default return to True """
        rmdir_flag = True
    else:
        """ As a safety feature against dangerous rm -fR
        commands, any directory path that does not begin
        with ~/ will be rejected (function returns False) """
        return False
    exists_dirpath = True
    with settings(warn_only=True):
        """ Tilde expansion only applies when '~' is unquoted """
        exists_dirpath = exists(tilde_expand(dirpath))
        if not exists_dirpath:
            return ignore_missing
    if exists_dirpath:
        if rmdir_flag:
            if rm_verbose:
                rm_line = "{0} -fR {1}".format('/bin/rm -v ',dirpath)
                run(rm_line)
            else:
                run("rm -fR " + dirpath)
        else:
            if rm_verbose:
                rm_line = "{0} -fR {1}".format('/bin/rm -v ',dirpath)
                run_or_sudo(rm_line)
            else:
                run_or_sudo("rm -fR " + dirpath)
    return rmdir_flag


def requires_binary_package(binary_unpathed=None,packagename_suggestion=None,
                            assumeyes=False):
    """ The word 'binary' here refers to the fact that you supply a binary
    name as the first argument. """
    which_res = None
    requires_which = '/bin/uname -r '
    with settings(warn_only=True):
        which_res = run("/usr/bin/which {0}".format(binary_unpathed))

    if which_res and which_res.succeeded:
        """ .succeeded is supported in Fabric 1.0 and newer """
        requires_which = which_res
        print which_res
        return requires_which
    elif which_res is not None and which_res.failed:
        if assumeyes:
            run_or_sudo("/usr/bin/apt-get --assume-yes install {0}".format(
                    packagename_suggestion))
        else:
            run_or_sudo("/usr/bin/apt-get install {0}".format(packagename_suggestion))

        with settings(warn_only=True):
            which_res = run("/usr/bin/which {0}".format(binary_unpathed))

        if which_res and not which_res.failed:
            requires_which = which_res
            print which_res
            return requires_which
    else:
        return which_res
    return False


def grep2():
    grep_module_target = "'django\.contrib\.|\.csrf\.|\.sessions|[a-z]*\.module\.[a-z]*'"
    # Last part of the above target is a form used by some Django cms
    grep_res = run_or_sudo("/bin/egrep -B1 -A1 {0} {1}/{2}/{2}/settings.py".format(
            grep_module_target,dirpath_fab,PROJECT_NAME))
    return


def default(repos=False):
    global pretty_name_id, pretty_name_word
    global dirpath_fab, dirpath_repos
    """ If you want fabric command line interaction with an argument
    then expect it to be a string. May need to int() what you get """
    pname = pretty_name()
    if not pname.endswith(chr(33)):
        pname_array = pname.split()
        pretty_name_id = pname_array[0].lower()
        if 'GNU' in pname:
            """ Last word """
            pretty_name_word = pname_array[-1].strip('()')
        else:
            """ 2nd word / middle word """
            pretty_name_word = pname_array[1]

    print "{0} {1}".format(pretty_name_id,pretty_name_word)

    rmdir_existing("~/{0}".format(dir_fab),True,True)
    dirpath_fab = tilde_expand("~/{0}".format(dir_fab))
    mkdir_res = run("mkdir -m 2755 {0}".format(dirpath_fab))
    if mkdir_res.succeeded:
        dirpath_dlp(dirpath_fab)
        #print_success_and_failed(mkdir_res,dirpath_fab)
    else:
        abort('Aborted during initialisation of directory.')
        #abort_with_cleanup(dirpath_fab,'Aborting during directory initialisation.')

    requires_django_res = requires_binary_package('django-admin',
                                                  'python-django',assumeyes=True)

    print "Creating project below directory {0}".format(dirpath_fab)
    with cd(dirpath_fab):
        startproject_res = run("django-admin startproject {0}".format(PROJECT_NAME))
        if startproject_res.succeeded:
            with cd(PROJECT_NAME):
                startapp_res = run("django-admin startapp {0}".format(APP_NAME))

    rmdir_existing("~/{0}.repos".format(dir_fab),True,True)
    dirpath_repos = tilde_expand("~/{0}.repos".format(dir_fab))
    mkdir_res = run("mkdir -m 2755 {0}".format(dirpath_repos))
    if repos:
        with cd(dirpath_repos):
            repo_clone(repo_url_django,'fd96e9c')
            repo_clone(repo_url_localbin,'0767ff5')
    grep2()
    return

