#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   Copyright 2013 Gary Wright wrightsolutions.co.uk/contact
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

# Tested against fabric 1.4.3
# http://ftp.uk.debian.org/debian/pool/main/f/fabric/fabric_1.4.3-1.debian.tar.gz

from __future__ import with_statement
from fabric.api import abort, cd, env, get, hide, put, run, settings, sudo
from fabric.contrib.console import confirm
from fabric.contrib.files import exists,append
from os import path
import re
try:
    from . import cmdbin
except ValueError:
    import cmdbin

""" res.succeeded is preferred for newer fabric, but not res.failed
is used where backward compatibility is preferable. """

CMD_RPM = "/bin/rpm "
CMD_RPM_INSTALL = "/bin/rpm -ivh "
CMD_RPM_TEST = "/bin/rpm --test "
CMD_YUM = "/usr/bin/yum "
CMD_YUM_ASSUMEYES = "/usr/bin/yum --assumeyes "
CMD_YUM_CLEAN_ALL = "/usr/bin/yum clean all "
CMD_YUM_REPOLIST = "/usr/bin/yum repolist "

def repolist_dict(reponame=None):
    if reponame is not None:
        repolist_res = run("{0} {1}".format(CMD_YUM_REPOLIST,reponame))
    else:
        with hide('output'):
            """ If no reponame given then yum can be quite chatty so hide() """
            run(CMD_YUM_CLEAN_ALL)
            repolist_res = run(CMD_YUM_REPOLIST)
    repolist_lines = cmdbin.run_res_lines_stripped(repolist_res)
    repo_dict = {}
    for line_unstripped in repolist_lines:
        line = line_unstripped.strip()
        if line.startswith('Load') or line.startswith('Determining'):
            pass
        elif line.startswith('repolist') or line.startswith('repo id'):
            pass
        elif line.lstrip().startswith('*'):
            if 'base:' in line:
                print line
            else:
                pass
        else:
            try:
                repo_line_array = line.split()
                label = repo_line_array[0]
                repo_dict[label]=' '.join(repo_line_array[1:])
            except Exception:
                raise
    return repo_dict

def has_epel():
    has_flag = False
    repolist = repolist_dict()
    if len(repolist) > 0 and 'epel' in repolist:
        has_flag = True
    return has_flag

